package api

import (
	"box-cost/db/model"
	"box-cost/db/repo"
	"box-cost/log"
	"errors"
	"fmt"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

// 供应商管理
func Supplier(r *GinRouter) {

	// 创建供应商
	r.POST("/supplier", CreateSupplier)

	// 获取供应商详情
	r.GET("/supplier/:id", GetSupplier)

	// 获取供应商列表
	r.GET("/suppliers", GetSuppliers)

	// 更新供应商
	r.POST("/supplier/update", UpdateSupplier)

	// 删除供应商
	r.POST("/supplier/delete/:id", DelSupplier)
}

// 创建供应商
func CreateSupplier(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	var supplier model.Supplier

	err := c.ShouldBindJSON(&supplier)
	if err != nil {
		fmt.Println(err)
		return nil, errors.New("参数错误！")
	}
	ctx := apictx.CreateRepoCtx()

	if supplier.Name == "" {
		return nil, errors.New("供应商名为空")
	}
	if supplier.Address == "" {
		return nil, errors.New("供应商地址为空")
	}
	if supplier.Phone == "" {
		return nil, errors.New("供应商联系电话为空")
	}

	supplier.CreateTime = time.Now()
	supplier.UpdateTime = time.Now()

	result, err := repo.RepoAddDoc(ctx, repo.CollectionSupplier, &supplier)
	return result, err
}

// 获取供应商信息
func GetSupplier(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	supplierId := c.Param("id")
	id, err := primitive.ObjectIDFromHex(supplierId)
	if err != nil {
		return nil, errors.New("非法id")
	}
	var supplier model.Supplier
	option := &repo.DocSearchOptions{
		CollectName: repo.CollectionSupplier,
		Query:       repo.Map{"_id": id},
	}

	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), option, &supplier)
	if !found || err != nil {
		log.Info(err)
		return nil, errors.New("数据未找到")
	}

	return supplier, nil
}

// 获取供应商列表
func GetSuppliers(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)

	option := &repo.PageSearchOptions{
		CollectName: repo.CollectionSupplier,
		Query:       query,
		Page:        page,
		Size:        size,
		Sort:        bson.M{"createTime": -1},
	}
	return repo.RepoPageSearch(apictx.CreateRepoCtx(), option)
}

// 更新供应商
func UpdateSupplier(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var supplier model.Supplier
	err := c.ShouldBindJSON(&supplier)
	if err != nil {
		return nil, errors.New("参数错误")
	}
	if supplier.Id.Hex() == "" {
		return nil, errors.New("id的为空")
	}
	supplier.UpdateTime = time.Now()
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionSupplier, supplier.Id.Hex(), &supplier)
}

// 删除供应商
func DelSupplier(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	supplierId := c.Param("id")
	if supplierId == "" {
		return nil, errors.New("id为空")
	}

	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionSupplier, supplierId)
}
